@@ -1,15 +1,11 @@
 #include "file.hh"
 
 #include "assert.hh"
-#include "buffer.hh"
-#include "exception.hh"
-#include "flags.hh"
-#include "option_types.hh"
 #include "event_manager.hh"
-#include "ranked_match.hh"
-#include "regex.hh"
 #include "string.hh"
-#include "unicode.hh"
+#include "string_utils.hh"
+#include "format.hh"
+#include "ranges.hh"
 
 #include <limits>
 #include <cerrno>
@@ -41,16 +37,12 @@
 namespace Kakoune
 {
 
-struct file_access_error : runtime_error
-{
-public:
-    file_access_error(StringView filename,
-                      StringView error_desc)
-        : runtime_error(format("{}: {}", filename, error_desc)) {}
+file_access_error::file_access_error(StringView filename,
+                                     StringView error_desc)
+    : runtime_error(format("{}: {}", filename, error_desc)) {}
 
-    file_access_error(int fd, StringView error_desc)
-        : runtime_error(format("fd {}: {}", fd, error_desc)) {}
-};
+file_access_error::file_access_error(int fd, StringView error_desc)
+    : runtime_error(format("fd {}: {}", fd, error_desc)) {}
 
 String parse_filename(StringView filename, StringView buf_dir)
 {
@@ -271,48 +263,38 @@ void write(int fd, StringView data)
             count -= written;
         }
         else if (errno == EAGAIN and not atomic and EventManager::has_instance())
-            EventManager::instance().handle_next_events(EventMode::Urgent, nullptr, false);
+            EventManager::instance().handle_next_events(EventMode::Urgent, nullptr, std::chrono::nanoseconds{});
         else
-            throw file_access_error(format("fd: {}", fd), strerror(errno));
+            throw file_access_error(fd, strerror(errno));
     }
 }
 template void write<true>(int fd, StringView data);
 template void write<false>(int fd, StringView data);
 
+int create_file(const char* filename)
+{
+    int fd;
+    const int flags = O_CREAT | O_WRONLY | O_TRUNC | (EventManager::has_instance() ? O_NONBLOCK : 0);
+    while ((fd = open(filename, flags, 0644)) == -1)
+    {
+        if (errno == ENXIO and EventManager::has_instance()) // trying to open a FIFO with no readers yet
+            EventManager::instance().handle_next_events(EventMode::Urgent, nullptr,
+                                                        std::chrono::nanoseconds{1'000'000});
+        else
+            return -1;
+    }
+    return fd;
+}
 
 void write_to_file(StringView filename, StringView data)
 {
-    const int fd = open(filename.zstr(), O_CREAT | O_WRONLY | O_TRUNC, 0644);
+    int fd = create_file(filename.zstr());
     if (fd == -1)
         throw file_access_error(filename, strerror(errno));
     auto close_fd = on_scope_end([fd]{ close(fd); });
     write(fd, data);
 }
 
-void write_buffer_to_fd(Buffer& buffer, int fd)
-{
-    auto eolformat = buffer.options()["eolformat"].get<EolFormat>();
-    StringView eoldata;
-    if (eolformat == EolFormat::Crlf)
-        eoldata = "\r\n";
-    else
-        eoldata = "\n";
-
-
-    BufferedWriter<false> writer{fd};
-    if (buffer.options()["BOM"].get<ByteOrderMark>() == ByteOrderMark::Utf8)
-        writer.write("\xEF\xBB\xBF");
-
-    for (LineCount i = 0; i < buffer.line_count(); ++i)
-    {
-        // end of lines are written according to eolformat but always
-        // stored as \n
-        StringView linedata = buffer[i];
-        writer.write(linedata.substr(0, linedata.length()-1));
-        writer.write(eoldata);
-    }
-}
-
 int open_temp_file(StringView filename, char (&buffer)[PATH_MAX])
 {
     String path = real_path(filename);
@@ -332,72 +314,6 @@ int open_temp_file(StringView filename)
     return open_temp_file(filename, buffer);
 }
 
-void write_buffer_to_file(Buffer& buffer, StringView filename,
-                          WriteMethod method, WriteFlags flags)
-{
-    auto zfilename = filename.zstr();
-    struct stat st;
-
-    bool replace = method == WriteMethod::Replace;
-    bool force = flags & WriteFlags::Force;
-
-    if ((replace or force) and (::stat(zfilename, &st) != 0 or
-                                (st.st_mode & S_IFMT) != S_IFREG))
-    {
-        force = false;
-        replace = false;
-    }
-
-    if (force and ::chmod(zfilename, st.st_mode | S_IWUSR) < 0)
-        throw runtime_error(format("unable to change file permissions: {}", strerror(errno)));
-
-    char temp_filename[PATH_MAX];
-    const int fd = replace ? open_temp_file(filename, temp_filename)
-                           : open(zfilename, O_CREAT | O_WRONLY | O_TRUNC, 0644);
-    if (fd == -1)
-    {
-        auto saved_errno = errno;
-        if (force)
-            ::chmod(zfilename, st.st_mode);
-        throw file_access_error(filename, strerror(saved_errno));
-    }
-
-    {
-        auto close_fd = on_scope_end([fd]{ close(fd); });
-        write_buffer_to_fd(buffer, fd);
-        if (flags & WriteFlags::Sync)
-            ::fsync(fd);
-    }
-
-    if (replace and geteuid() == 0 and ::chown(temp_filename, st.st_uid, st.st_gid) < 0)
-        throw runtime_error(format("unable to set replacement file ownership: {}", strerror(errno)));
-    if (replace and ::chmod(temp_filename, st.st_mode) < 0)
-        throw runtime_error(format("unable to set replacement file permissions: {}", strerror(errno)));
-    if (force and not replace and ::chmod(zfilename, st.st_mode) < 0)
-        throw runtime_error(format("unable to restore file permissions: {}", strerror(errno)));
-
-    if (replace and rename(temp_filename, zfilename) != 0)
-    {
-        if (force)
-            ::chmod(zfilename, st.st_mode);
-        throw runtime_error("replacing file failed");
-    }
-
-    if ((buffer.flags() & Buffer::Flags::File) and
-        real_path(filename) == real_path(buffer.name()))
-        buffer.notify_saved(get_fs_status(real_path(filename)));
-}
-
-void write_buffer_to_backup_file(Buffer& buffer)
-{
-    const int fd = open_temp_file(buffer.name());
-    if (fd >= 0)
-    {
-        write_buffer_to_fd(buffer, fd);
-        close(fd);
-    }
-}
-
 String find_file(StringView filename, StringView buf_dir, ConstArrayView<String> paths)
 {
     struct stat buf;
@@ -450,18 +366,16 @@ void make_directory(StringView dir, mode_t mode)
     }
 }
 
-template<MemoryDomain domain = MemoryDomain::Undefined, typename Filter>
-Vector<String, domain> list_files(StringView dirname, Filter filter)
+void list_files(StringView dirname, FunctionRef<void (StringView, const struct stat&)> callback)
 {
     char buffer[PATH_MAX+1];
     format_to(buffer, "{}", dirname);
     DIR* dir = opendir(dirname.empty() ? "./" : buffer);
     if (not dir)
-        return {};
+        return;
 
     auto close_dir = on_scope_end([dir]{ closedir(dir); });
 
-    Vector<String, domain> result;
     while (dirent* entry = readdir(dir))
     {
         StringView filename = entry->d_name;
@@ -471,147 +385,13 @@ Vector<String, domain> list_files(StringView dirname, Filter filter)
         struct stat st;
         auto fmt_str = (dirname.empty() or dirname.back() == '/') ? "{}{}" : "{}/{}";
         format_to(buffer, fmt_str, dirname, filename);
-        if (stat(buffer, &st) != 0 or not filter(*entry, st))
+        if (stat(buffer, &st) != 0)
             continue;
 
         if (S_ISDIR(st.st_mode))
             filename = format_to(buffer, "{}/", filename);
-        result.push_back(filename.str());
+        callback(filename, st);
     }
-    return result;
-}
-
-template<MemoryDomain domain = MemoryDomain::Undefined>
-Vector<String, domain> list_files(StringView directory)
-{
-    return list_files(directory, [](const dirent& entry, const struct stat&) {
-                          return StringView{entry.d_name}.substr(0_byte, 1_byte) != ".";
-                      });
-}
-
-Vector<String> list_files(StringView directory)
-{
-    return list_files<>(directory);
-}
-
-static CandidateList candidates(ConstArrayView<RankedMatch> matches, StringView dirname)
-{
-    CandidateList res;
-    res.reserve(matches.size());
-    for (auto& match : matches)
-        res.push_back(dirname + match.candidate());
-    return res;
-}
-
-CandidateList complete_filename(StringView prefix, const Regex& ignored_regex,
-                                ByteCount cursor_pos, FilenameFlags flags)
-{
-    prefix = prefix.substr(0, cursor_pos);
-    auto [dirname, fileprefix] = split_path(prefix);
-    auto parsed_dirname = parse_filename(dirname);
-
-    Optional<ThreadedRegexVM<const char*, RegexMode::Forward | RegexMode::AnyMatch | RegexMode::NoSaves>> vm;
-    if (not ignored_regex.empty())
-    {
-        vm.emplace(*ignored_regex.impl());
-        if (vm->exec(fileprefix.begin(), fileprefix.end(), fileprefix.begin(), fileprefix.end(), RegexExecFlags::None))
-            vm.reset();
-    }
-
-    const bool only_dirs = (flags & FilenameFlags::OnlyDirectories);
-
-    auto filter = [&vm, only_dirs](const dirent& entry, struct stat& st)
-    {
-        StringView name{entry.d_name};
-        return (not vm or not vm->exec(name.begin(), name.end(), name.begin(), name.end(), RegexExecFlags::None)) and
-               (not only_dirs or S_ISDIR(st.st_mode));
-    };
-    auto files = list_files(parsed_dirname, filter);
-    Vector<RankedMatch> matches;
-    for (auto& file : files)
-    {
-        if (RankedMatch match{file, fileprefix})
-            matches.push_back(match);
-    }
-    // Hack: when completing directories, also echo back the query if it
-    // is a valid directory. This enables menu completion to select the
-    // directory instead of a child.
-    if (only_dirs and not dirname.empty() and dirname.back() == '/' and fileprefix.empty()
-        and /* exists on disk */ not files.empty())
-    {
-        matches.push_back(RankedMatch{fileprefix, fileprefix});
-    }
-    std::sort(matches.begin(), matches.end());
-    const bool expand = (flags & FilenameFlags::Expand);
-    return candidates(matches, expand ? parsed_dirname : dirname);
-}
-
-CandidateList complete_command(StringView prefix, ByteCount cursor_pos)
-{
-    String real_prefix = parse_filename(prefix.substr(0, cursor_pos));
-    auto [dirname, fileprefix] = split_path(real_prefix);
-
-    if (not dirname.empty())
-    {
-        auto filter = [](const dirent& entry, const struct stat& st)
-        {
-            bool executable = (st.st_mode & S_IXUSR)
-                            | (st.st_mode & S_IXGRP)
-                            | (st.st_mode & S_IXOTH);
-            return S_ISDIR(st.st_mode) or (S_ISREG(st.st_mode) and executable);
-        };
-        auto files = list_files(dirname, filter);
-        Vector<RankedMatch> matches;
-        for (auto& file : files)
-        {
-            if (RankedMatch match{file, fileprefix})
-                matches.push_back(match);
-        }
-        std::sort(matches.begin(), matches.end());
-        return candidates(matches, dirname);
-    }
-
-    using TimeSpec = decltype(stat::st_mtim);
-
-    struct CommandCache
-    {
-        TimeSpec mtim = {};
-        Vector<String, MemoryDomain::Completion> commands;
-    };
-    static HashMap<String, CommandCache, MemoryDomain::Commands> command_cache;
-
-    Vector<RankedMatch> matches;
-    for (auto dir : StringView{getenv("PATH")} | split<StringView>(':'))
-    {
-        auto dirname = ((not dir.empty() and dir.back() == '/') ? dir.substr(0, dir.length()-1) : dir).str();
-
-        struct stat st;
-        if (stat(dirname.c_str(), &st))
-            continue;
-
-        auto& cache = command_cache[dirname];
-        if (memcmp(&cache.mtim, &st.st_mtim, sizeof(TimeSpec)) != 0)
-        {
-            auto filter = [](const dirent& entry, const struct stat& st) {
-                bool executable = (st.st_mode & S_IXUSR)
-                                | (st.st_mode & S_IXGRP)
-                                | (st.st_mode & S_IXOTH);
-                return S_ISREG(st.st_mode) and executable;
-            };
-
-            cache.commands = list_files<MemoryDomain::Completion>(dirname, filter);
-            memcpy(&cache.mtim, &st.st_mtim, sizeof(TimeSpec));
-        }
-        for (auto& cmd : cache.commands)
-        {
-            if (RankedMatch match{cmd, fileprefix})
-                matches.push_back(match);
-        }
-    }
-    std::sort(matches.begin(), matches.end());
-    auto it = std::unique(matches.begin(), matches.end());
-    matches.erase(it, matches.end());
-    return candidates(matches, "");
 }
 
 timespec get_fs_timestamp(StringView filename)
