@@ -1,12 +1,13 @@
 #include "assert.hh"
 #include "backtrace.hh"
 #include "buffer.hh"
-#include "buffer_manager.hh"
 #include "buffer_utils.hh"
+#include "buffer_manager.hh"
 #include "client_manager.hh"
 #include "command_manager.hh"
 #include "commands.hh"
 #include "context.hh"
+#include "debug.hh"
 #include "event_manager.hh"
 #include "face_registry.hh"
 #include "file.hh"
@@ -27,7 +28,6 @@
 #include "string.hh"
 #include "unit_tests.hh"
 #include "window.hh"
-#include "clock.hh"
 
 #include <fcntl.h>
 #include <locale.h>
@@ -45,6 +45,12 @@ struct {
     unsigned int version;
     StringView notes;
 } constexpr version_notes[] = { {
+        20250603,
+        "» kak_* appearing in shell arguments will be added to the environment\n"
+        "» {+U}double underline{} support\n"
+        "» {+u}git apply{} can stage/revert selected changes to current buffer\n"
+        "» {+u}exec/eval -client{} accepts '*' and comma separated list\n"
+    }, {
         20240518,
         "» Fix tests failing on some platforms\n"
     }, {
@@ -591,6 +597,7 @@ void register_options()
                        "    terminal_assistant             clippy|cat|dilbert|none|off\n"
                        "    terminal_status_on_top         bool\n"
                        "    terminal_set_title             bool\n"
+                       "    terminal_title                 str\n"
                        "    terminal_enable_mouse          bool\n"
                        "    terminal_synchronized          bool\n"
                        "    terminal_wheel_scroll_amount   int\n"
@@ -683,21 +690,12 @@ pid_t fork_server_to_background()
 
 std::unique_ptr<UserInterface> create_local_ui(UIType ui_type)
 {
-    if (ui_type == UIType::Terminal and not isatty(0))
-    {
-        // move stdin to another fd, and restore tty as stdin
-        int fd = dup(0);
-        int tty = open("/dev/tty", O_RDONLY);
-        dup2(tty, 0);
-        close(tty);
-        create_fifo_buffer("*stdin*", fd, Buffer::Flags::None);
-    }
-
     auto ui = make_ui(ui_type);
 
     static SignalHandler old_handler = set_signal_handler(SIGTSTP, [](int sig) {
         if (ClientManager::instance().count() == 1 and
-            *ClientManager::instance().begin() == local_client)
+            *ClientManager::instance().begin() == local_client and
+            not Server::instance().is_daemon())
             old_handler(sig);
         else
         {
@@ -825,11 +823,23 @@ int run_server(StringView session, StringView server_init,
                                      "    {}", error.what()));
     }
 
+    {
+        Context empty_context{Context::EmptyContextFlag{}};
+        global_scope.hooks().run_hook(Hook::EnterDirectory, real_path("."), empty_context);
+        global_scope.hooks().run_hook(Hook::KakBegin, session, empty_context);
+    }
+
     if (not server_init.empty()) try
     {
         Context init_context{Context::EmptyContextFlag{}};
         command_manager.execute(server_init, init_context);
     }
+    catch (const kill_session& kill)
+    {
+        Context empty_context{Context::EmptyContextFlag{}};
+        global_scope.hooks().run_hook(Hook::KakEnd, "", empty_context);
+        return kill.exit_status;
+    }
     catch (runtime_error& error)
     {
         startup_error = true;
@@ -837,11 +847,6 @@ int run_server(StringView session, StringView server_init,
                                      "    {}", error.what()));
     }
 
-    {
-        Context empty_context{Context::EmptyContextFlag{}};
-        global_scope.hooks().run_hook(Hook::KakBegin, session, empty_context);
-    }
-
     if (not files.empty()) try
     {
         for (auto& file : files)
@@ -871,6 +876,16 @@ int run_server(StringView session, StringView server_init,
     int exit_status = 0;
     try
     {
+        if (ui_type == UIType::Terminal and not isatty(0))
+        {
+            // move stdin to another fd, and restore tty as stdin
+            int fd = dup(0);
+            int tty = open("/dev/tty", O_RDONLY);
+            dup2(tty, 0);
+            close(tty);
+            create_fifo_buffer("*stdin*", fd, Buffer::Flags::None, AutoScroll::NotInitially);
+        }
+
         if (not server.is_daemon())
         {
             local_client = client_manager.create_client(
@@ -894,14 +909,16 @@ int run_server(StringView session, StringView server_init,
 
             // Loop so that eventual inputs happening during the processing are handled as
             // well, avoiding unneeded redraws.
-            bool allow_blocking = not client_manager.has_pending_inputs();
+            Optional<std::chrono::nanoseconds> timeout;
+            if (client_manager.has_pending_inputs())
+                timeout = std::chrono::nanoseconds{};
             try
             {
-                while (event_manager.handle_next_events(EventMode::Normal, nullptr, allow_blocking))
+                while (event_manager.handle_next_events(EventMode::Normal, nullptr, timeout))
                 {
                     if (client_manager.process_pending_inputs())
                         break;
-                    allow_blocking = false;
+                    timeout = std::chrono::nanoseconds{};
                 }
             }
             catch (const cancel&) {}
@@ -1138,14 +1155,16 @@ int main(int argc, char* argv[])
         const bool clear_sessions = (bool)parser.get_switch("clear");
         if (list_sessions or clear_sessions)
         {
-            for (auto& session : list_files(session_directory()))
-            {
+            list_files(session_directory(), [&](StringView session, auto&) {
+                if (session.substr(0_byte, 1_byte) == ".")
+                    return;
+
                 const bool valid = check_session(session);
                 if (list_sessions)
                     write_stdout(format("{}{}\n", session, valid ? "" : " (dead)"));
                 if (not valid and clear_sessions)
                     unlink(session_path(session).c_str());
-            }
+            });
             return 0;
         }
 
